/*
 * Copyright 2016 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.kie.workbench.common.forms.fields.shared.fieldTypes.basic.slider.provider;

import javax.enterprise.context.Dependent;

import org.kie.workbench.common.forms.fields.shared.fieldTypes.basic.BasicTypeFieldProvider;
import org.kie.workbench.common.forms.fields.shared.fieldTypes.basic.slider.definition.DoubleSliderDefinition;
import org.kie.workbench.common.forms.fields.shared.fieldTypes.basic.slider.definition.IntegerSliderDefinition;
import org.kie.workbench.common.forms.fields.shared.fieldTypes.basic.slider.definition.SliderBaseDefinition;
import org.kie.workbench.common.forms.fields.shared.fieldTypes.basic.slider.type.SliderFieldType;
import org.kie.workbench.common.forms.model.FieldDataType;

@Dependent
public class SliderFieldProvider extends BasicTypeFieldProvider<SliderBaseDefinition> {

    @Override
    public Class<SliderFieldType> getFieldType() {
        return SliderFieldType.class;
    }

    @Override
    public String getFieldTypeName() {
        return SliderBaseDefinition.FIELD_TYPE.getTypeName();
    }

    @Override
    protected void doRegisterFields() {
        registerPropertyType(Number.class);
        registerPropertyType(Double.class);
        registerPropertyType(double.class);
        registerPropertyType(Integer.class);
        registerPropertyType(int.class);
    }

    @Override
    public int getPriority() {
        return 5;
    }

    @Override
    public SliderBaseDefinition getDefaultField() {
        return new DoubleSliderDefinition();
    }

    @Override
    public SliderBaseDefinition createFieldByType(FieldDataType typeInfo) {
        if (Integer.class.getName().equals(typeInfo.getType()) ||
                int.class.getName().equals(typeInfo.getType())) {
            return new IntegerSliderDefinition();
        }
        return new DoubleSliderDefinition();
    }
}
